/*
 * Copyright 2005 Invoq Systems Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are subject to an established license agreement with 
 * Invoq Systems Inc. 
 * 
 * This work is the intellectual property of Invoq Systems Inc. and 
 * is subject to US and International Copyright Law.
 * 
 * Work Created: Jun 13, 2005
 */
package com.invoqsystems.foundation.log;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.net.SocketTimeoutException;

import org.apache.log4j.Logger;
import org.aspectj.lang.reflect.MethodSignature;
import org.aspectj.lang.Signature;

public aspect ExceptionLogging 
{
    private Logger exceptionLogger = Logger.getLogger(ExceptionLogging.class);

    pointcut exceptionHandlers(Throwable t) : handler(Throwable+) && (args(t)) && !within(org.apache.activemq..*);

    before(Throwable t) : exceptionHandlers(t) 
    {
        // exclude handlers in methods annotated by @SuppressExceptionLogging;
        // for non-methods (e.g., constructors), there is no suppression
        Signature signature = thisEnclosingJoinPointStaticPart.getSignature();
        
        if (signature instanceof MethodSignature)
        {
        	Method enclosingMethod = ((MethodSignature) signature).getMethod();
	        if(enclosingMethod.isAnnotationPresent(SuppressExceptionLogging.class))
    	        return;
        }
        
        exceptionLogger.error(t.toString(), t);
    }
}
